# -*- coding: utf-8 -*-
"""
Loader submodule

Contains all methods used to load an edX course (XML and policy)
"""
from olxcleaner.loader.xml import load_course
from olxcleaner.loader.policy import load_policy
