# -*- coding: utf-8 -*-
"""
entries submodule

Contains files for each external entry, as defined in setup.py.

This submodule is not intended for external use. Rather, it contains
light wrappers around the olxcleaner package.
"""
